@@ -1,12 +1,12 @@
 /* MTX -- SCSI Tape Attached Medium Control Program
 
-   Copyright 1997-1998 Leonard N. Zubkoff <lnz@dandelion.com>
- 
-   Changes 1999 Eric Lee Green to add support for multi-drive tape changers.
+	Copyright 1997-1998 Leonard N. Zubkoff <lnz@dandelion.com>
 
-   $Date: 2003/02/21 17:45:23 $
-   $Revision: 1.7 $
-   See mtx.c for licensing information. 
+	Changes 1999 Eric Lee Green to add support for multi-drive tape changers.
+
+	$Date: 2007-03-04 15:27:11 -0800 (Sun, 04 Mar 2007) $
+	$Revision: 164 $
+	See mtx.c for licensing information. 
 
 */
 
@@ -18,7 +18,11 @@
 #include "[.vms]defs.h"
 #else /* all the Unix stuff:  */
 
+#ifdef _MSC_VER
+#include "msvc/config.h"  /* all the autoconf stuff. */
+#else
 #include "config.h"  /* all the autoconf stuff. */
+#endif
 
 /* all the general Unix includes: */
 
@@ -59,7 +63,11 @@
 #  include <sys/ioctl.h>
 #endif
 
-/* Now greately modified to use GNU Autoconf stuff: */
+#if HAVE_SYS_PARAM_H
+#  include <sys/param.h>
+#endif
+
+/* Now greatly modified to use GNU Autoconf stuff: */
 /* If we use the 'sg' interface, like Linux, do this: */
 #if HAVE_SCSI_SG_H
 #  include <scsi/scsi.h>
@@ -69,6 +77,25 @@ typedef int DEVICE_TYPE; /* the sg interface uses this. */
 #  define HAVE_GET_ID_LUN 1  /* signal that we have it... */
 #endif
 
+/* Windows Native programs built using MinGW */
+#if HAVE_DDK_NTDDSCSI_H
+#  include <windows.h>
+#  include <ddk/ntddscsi.h>
+#  undef DEVICE_TYPE
+
+typedef int DEVICE_TYPE;
+#endif
+
+/* Windows Native programs built using Microsoft Visual C */
+#ifdef _MSC_VER
+#  include <windows.h>
+#  include <winioctl.h>
+#  include <ntddscsi.h>
+#  undef DEVICE_TYPE
+
+typedef int DEVICE_TYPE;
+#endif
+
 /* The 'cam' interface, like FreeBSD: */
 #if HAVE_CAMLIB_H
 #  include <camlib.h> /* easy (?) access to the CAM user library. */
@@ -160,26 +187,42 @@ typedef dsreq_t *DEVICE_TYPE; /* 64-bit pointers/32bit int on later sgi? */
 #define MAX_TRANSFER_ELEMENTS 2  /* we just do dual-drive for now :-} */
 #define MAX_TRANSPORT_ELEMENTS 1 /* we just do one arm for now... */
 
+#define MTX_ELEMENTSTATUS_ORIGINAL 0
+#define MTX_ELEMENTSTATUS_READALL 1
+
 /* These are flags used for the READ_ELEMENT_STATUS and MOVE_MEDIUM
  * commands:
  */
-typedef struct SCSI_Flags_Struct {
-  unsigned char eepos;
-  unsigned char invert;
-  unsigned char no_attached; /* ignore _attached bit */
-  unsigned char no_barcodes;  /* don't try to get barcodes. */
-  int numbytes;
-  int elementtype;
-  int numelements;
-  int attached;
-  int has_barcodes;
-  unsigned char invert2; /* used for EXCHANGE command, sigh. */
-} SCSI_Flags_T;
+typedef struct SCSI_Flags_Struct
+{
+	unsigned char eepos;
+	unsigned char invert;
+	unsigned char no_attached; /* ignore _attached bit */
+	unsigned char no_barcodes;  /* don't try to get barcodes. */
+	int numbytes;
+	int elementtype;
+	int numelements;
+	int attached;
+	int has_barcodes;
+	int querytype; //MTX_ELEMENTSTATUS
+	unsigned char invert2; /* used for EXCHANGE command, sigh. */
+}	SCSI_Flags_T;
 
+#ifdef _MSC_VER
+typedef unsigned char boolean;
+
+#define false   0
+#define true    1
+
+typedef unsigned char Direction_T;
+
+#define Input   0
+#define Output  1
+#else
 typedef enum { false, true } boolean;
 
-
 typedef enum { Input, Output } Direction_T;
+#endif
 
 
 typedef unsigned char CDB_T[12];
@@ -354,6 +397,15 @@ typedef struct ElementModeSenseHeader {
 } ElementModeSense_T;
 
 
+#ifdef _MSC_VER
+typedef char ElementTypeCode_T;
+
+#define AllElementTypes	        0
+#define MediumTransportElement  1
+#define StorageElement	        2
+#define ImportExportElement     3
+#define DataTransferElement     4
+#else
 typedef enum ElementTypeCode
 {
   AllElementTypes =		0,
@@ -363,6 +415,7 @@ typedef enum ElementTypeCode
   DataTransferElement =		4
 }
 ElementTypeCode_T;
+#endif
 
 
 typedef struct ElementStatusDataHeader
